--{{{ _Yank Map Voting Script
if not mp then mp = {maps = {}} end
mp.config = {
	polltime = 60;					-- Time (in seconds) for each poll
	lastsecs = 30; 					-- Extra time (in seconds) for the map change when a poll succeeds
	extra_no = 0;					-- The balance between the NO's and YES's. Set to 0 to disable
	cooldown = 90;					-- The time (in seconds) the player needs to wait to start a vote poll again
	hook_domination = 2;			-- How is the serveraction going to be controlled (mode 1, through log hook, mode 2 through addhook function)
}

function mp.mapsupdate()
	mp.maps = {}
	local file = io.open("sys/lua/maplist.cfg","r")
	for line in file:lines() do if line ~= "" then table.insert(mp.maps, line) end end
	file:close()
end

timer(30, "mp.SETUP")
function mp.SETUP()
	if bass then
		mp.ABOUT = {
			name = "Map Voting";
			author = "_Yank";
			version = "1.7b";
			commands = {{"votemap", level = 0, func = "mp.mapmenu", desc = "Vote for a map."}};
		}
	else
		addhook("say", "mp.say", -999)
		function mp.say(id, txt) if txt == "!votemap" then mp.mapmenu(id, 1) return 1 end end
	end
end

mp.huds = {
	{48, 5, 115};
	{49, 5, 130};
}

mp.hooks = {}
mp.done = false

local debuglog = game("debuglog")
local debuglua = game("debuglua")

parse("debuglua 1")
parse("debuglog 1")

mp.originalAddhook = addhook
if mp.config.hook_domination == 1 then
	addhook("log", "mp.avoid")
	function mp.avoid(text)
		if not mp.done and string.sub(text, 1, 20) == "Lua: Adding function" then
			local func = text:match("'(.*)' to hook 'serveraction'")
			if func and func ~= "mp.serveraction" then table.insert(mp.hooks, func) end
		end
		
		if mp.done and string.sub(text, 1, 20) == "Lua: Adding function" then return 1 end
	end
else
	function addhook(hook, ...)
		local arg = {...}

		if arg[1] ~= "mp.serveraction" and hook == "serveraction" then
			local prio = arg[2] or #mp.hooks + 1
			table.insert(mp.hooks, {arg[1], prio})
			table.sort(mp.hooks, function(a, b) return a[2] > b[2] end)
		end

		mp.originalAddhook(hook, ...)
	end
end

timer(800, "mp.recover")
function mp.recover()
	parse("debuglua "..debuglua)
	parse("debuglog "..debuglog)
end

mp.YES = 0
mp.NO = 0

mp.cooldown = mp.config.cooldown
timer(1000, "mp.clock", 0, mp.cooldown)
function mp.clock() mp.cooldown = mp.cooldown - 1 end

mp.voted = {}
mp.pcount = game("sv_maxplayers")
for id = 1, mp.pcount do mp.voted[id] = false end

function mp.mapmenu(id, page)
	mp.mapsupdate()

	local page = page or 1
	local pages = math.ceil(#mp.maps / 6)
	if page < 1 then page = pages end
	if page > pages then page = 1 end
	local menuString = "Maps ("..page.."/"..pages..")"
	for i = 6 * page - 5, 6 * page do
		if mp.maps[i] then menuString = menuString ..", ".. mp.maps[i] else menuString = menuString .."," end
	end
	if page == pages then menuString = menuString ..",,First page" else menuString = menuString ..",,Next page" end
	if page == 1 then menuString = menuString ..",Last page" else menuString = menuString ..",Previous page" end
	menu(id, menuString)
end

function mp.startpoll(id,map)
	if game("sv_map") == mp.maps[map] then msg2(id,"\169255000000Error, the map you voted is already running on the server")
	elseif mp.cooldown > 0 then msg2(id,"\169255000000Error, you need to wait more "..mp.cooldown.." second/s.")
	else
		local name = player(id,"name")
		mp.name = name
		mp.map = mp.maps[map]

		msg("\169000255000"..name.." called a vote for "..mp.map)
		timer(20000, "parse", "sv_msg \169000255000Press [F3] for YES,  [F4] for NO!", 2)
		msg("\169000255000Press [F3] for YES,  [F4] for NO!")
		msg("\169000255000"..player(id,"name").." voted YES!")

		mp.YES = mp.YES + 1
		mp.voted[id] = 1

		parse("hudtxt "..mp.huds[2][1].." \"\169000255000"..name.." called a vote for "..mp.map.."\" "..table.concat(mp.huds[2], " ", 2))
		mp.pollstarted = true
		
		mp.countdown = mp.config.polltime
		timer(1000, "mp.updatepoll", 0, mp.countdown)
		
		for id, hook in ipairs(mp.hooks) do freehook("serveraction", (type(hook) == "string" and hook) or hook[1]) end
	end
end

function mp.updatepoll()
	mp.countdown = mp.countdown - 1
	
	for id in ipairs(mp.voted) do
		if mp.voted[id] and not player(id, "exists") then
			if mp.voted[id] == 1 then mp.YES = mp.YES - 1
			else mp.NO = mp.NO - 1 end
			mp.voted[id] = false
		end
	end

	parse("hudtxt "..mp.huds[2][1].." \"\169000255000"..mp.name.." called a vote for "..mp.map.."\" "..table.concat(mp.huds[2], " ", 2))
	parse("hudtxt "..mp.huds[1][1].." \"\169218238075Time: "..mp.countdown.." YES: "..mp.YES.."  NO: "..mp.NO.."\" "..table.concat(mp.huds[1], " ", 2))
	if mp.countdown <= 0 then mp.performpoll() end
end

function mp.performpoll()
	if mp.YES > mp.NO + mp.config.extra_no then
		msg("\169000255000The poll is ACCEPTED!")
		mp.clearHuds()
		
		mp.change = mp.config.lastsecs + 0
		timer(1000, "mp.changemap", 0, mp.change)
		return
	elseif mp.YES < mp.NO + mp.config.extra_no then
		msg("\169255000000The poll is REJECTED!")
		msg("\169255255255More "..(mp.NO + mp.config.extra_no + 1) - mp.YES.." YES needed!")
	elseif mp.YES == mp.NO + mp.config.extra_no then
		msg("\169255255255The poll is DRAW! (Needed more 1 YES)")
	end
	
	mp.clearHuds()
	
	mp.pollstarted = false
	mp.YES = 0
	mp.NO = 0

	mp.cooldown = mp.config.cooldown
	timer(1000, "mp.clock", 0, mp.cooldown)

	for id = 1, mp.pcount do mp.voted[id] = false end
	mp.done = true
	for id, hook in ipairs(mp.hooks) do mp.originalAddhook("serveraction", (type(hook) == "string" and hook) or unpack(hook)) end
	mp.done = false
end

function mp.changemap()
	mp.change = mp.change - 1
	parse("hudtxt "..mp.huds[1][1].." \"\169000255000Mapchange to "..mp.map.." in "..mp.change.." seconds\" "..table.concat(mp.huds[1], " ", 2))
	if mp.change <= 0 then parse("map "..mp.map) end
end

function mp.clearHuds()
	parse("hudtxt "..mp.huds[1][1].." \"\" 0 0")
	parse("hudtxt "..mp.huds[2][1].." \"\" 0 0")
end

addhook("serveraction","mp.serveraction", -10000)
function mp.serveraction(id,action)
	if mp.pollstarted then
		action = tonumber(action)
		if not mp.voted[id] then
			if action == 1 then
				msg2(id,"\169000255000Vote first (F3 for YES, F4 for NO)!")
			elseif action == 2 then
				mp.YES = mp.YES + 1
				msg("\169000255000"..player(id,"name").." voted YES!")
				mp.voted[id] = 1
			elseif action == 3 then
				mp.NO = mp.NO + 1
				msg("\169255000000"..player(id,"name").." voted NO!")
				mp.voted[id] = 0
			end
		else
			for id, func in ipairs(mp.hooks) do
				loadstring(((type(func) == "string" and func) or func[1]).."(...)")(id, action)
			end
		end
	end
end

addhook("menu", "mp.menu")
function mp.menu(id, title, button)
	if title:sub(1, 4) == "Maps" then
		local page = title:match("(%d*)/"..math.ceil(#mp.maps / 6))
		local pages = math.ceil(#mp.maps / 6)
		if button == 8 then mp.mapmenu(id, page + 1) end
		if button == 9 then mp.mapmenu(id, page - 1) end
		if button <= 6 and button > 0 then
			if not mp.pollstarted then
				local map = ((page - 1) * 6) + button
				mp.startpoll(id, map)
			else msg2(id,"\169255000000Error, there's already a map vote pending!") end
		end
	end
end

mp.mapsupdate()